/*
Title: _WindowPadX

*Handling windows in general and within a multi-monitor setup in special*
    
derived from *WindowPad* by *Lexikos* (http://www.autohotkey.com/forum/topic21703.html)

See also the documentation of original *WindowPad* by *Lexikos* (http://www.autohotkey.com/forum/topic21703.html), as *WindowPadX* is just a simple clone of *WindowPad* with reengineering and a few enhancements ....

Documentation:
- <Command-line Usage> - Using *WindowPadX* from the commandline
- <Adding Commands> - Implement your own commands
    
Requires:
    AutoHotkey v1.1.0 or later.

Author: 
    hoppfrosch
 
Version History:
    1.2.2 - 22.Mar.2012 Hoppfrosch
        [+] WPXA.ahk v0.2.0: <wp_RollWindowToggle>: Bugfix for rolling up windows to its titlebar
    1.2.1 - 22.Mar.2012 Hoppfrosch
        [+] WPXA.ahk v0.1.10: <wp_GetMonitorFromMouse>: Determine monitor where mouse is
        [*] WPXA.ahk v0.1.10: <WPXA_MinimizeWindowsOnMonitor>: Minimize windows on screen where mouse is
    1.2.0 - 19.Mar.2012 Hoppfrosch
        [+] WPXA.ahk v0.1.9: <WPXA_TileLast2Windows>: Tile active and last window (Credits: ipstone today (http://www.autohotkey.com/forum/viewtopic.php?p=521482#521482))
    1.1.1 - 26.Jan.2012 Hoppfrosch
        [*] WPXA.ahk v0.1.8: <WPXA_MaximizeToggle>: Bugfix to actually toggle Maximization
    1.1.0 - 13.Jan.2012 Hoppfrosch
        [*] WPXA.ahk v0.1.6: <WPXA_TopToggle()>: Reanimated Notifications
        [+] WPXA.ahk v0.1.7: <WPXA_RollToggle>: New action for toggling rolling a window to its captionbar
    1.0.1 - 13.Jan.2012 Hoppfrosch
        [*] WPXA.ahk v0.1.5: <WPXA_MouseLocator>: Using integer coordinates for Gui Show
    1.0.0 - 12.Jan.2012 Hoppfrosch: Initial release of WindowPadX
*/

/*

--------------------------------------------------------------------------------------
Ideensammlung:
* Transparenz fr Fenster
* Overlay-Icon in Taskbar, um anzuzeigen auf welchem Screen sich das Fenster befindet ... (Funktion aus ITaskBar von maul.esel). Dies sollte bei verlassen des Programmes auch wieder entfernt werden.
  Hinweise:
  * http://www.autohotkey.com/forum/viewtopic.php?t=74314
  * http://www.autohotkey.com/forum/viewtopic.php?t=70978
* "TaskSwitcher" fuer jeden Monitor
  Hinweise:
  * http://www.autohotkey.com/forum/viewtopic.php?t=71912
-------------------------------------------------------------------------------------- 
*/

#include %A_ScriptDir%/_inc/WPXA.ahk

#SingleInstance force

Version := "1.2.2"

if 0 > 0
{
    ; Command-line mode: interpret each arg as a pseudo-command.
    ; Suspend all hotkeys which may be created by WindowPadXInit.
    Suspend On
    ; Load options and Gather exclusions.
    gosub WindowPadXInit
    ; Execute command line(s).  Each args should be in one of these formats:
    ;   <command>
    ;   <command>,<args_no_spaces>
    ;   "<command>, <args>"    ; In this case the initial comma is optional.
    Loop %0%
        wp_ExecLine(%A_Index%)
    ExitApp
}

OnExit, TrayExit

WindowPadXInit:
    ; If this script is #included in another script, this section may not be
    ; auto-executed.  In that case, the following should be called manually:
	WindowPadX_Init(WINDOWPADX_INI_PATH)
	;_________________________________________________
	;_______User Settings_____________________________
	; Make customisation only in this area or hotkey area only!!
	; The percentage by which to raise or lower the transparency each time:
	Transparency = 255
	Transparency_Step = 10

	; How long to display the Transparency level bar graphs:
	vol_DisplayTime = 2000

	; Transparency Bar color (see the help file to use more
	; precise shades):
	Transparency_CBM = Red

	; Background color
	Transparency_CW = Silver

	; Bar's screen position. Use -1 to center the bar in that dimension:

	Transparency_PosX = -1
	Transparency_PosY = -1
	Transparency_Width = 150 ; width of bar
	Transparency_Thick = 12 ; thickness of bar

	; try changing the below hotkeys to use them by specifying

	HotKey, !WheelUp, Transparency_Up
	HotKey, !WheelDown, Transparency_Down

	;___________________________________________
	;_____Auto Execute Section__________________
	; DON'T CHANGE ANYTHING HERE (unless you know what you're doing).

	Transparency_BarOptions = 1:B ZH%Transparency_Thick% ZX0 ZY0 W%Transparency_Width% CB%Transparency_CBM% CW%Transparency_CW%

	; If the X position has been specified, add it to the options.
	; Otherwise, omit it to center the bar horizontally:

	if Transparency_PosX >= 0
	{
		Transparency_BarOptions = %Transparency_BarOptions% X%Transparency_PosX%
	}

	; If the Y position has been specified, add it to the options.
	; Otherwise, omit it to have it calculated later:
	if Transparency_PosY >= 0
	{
		Transparency_BarOptions = %Transparency_BarOptions% Y%Transparency_PosY%
	}
	#SingleInstance
	SetBatchLines, 10ms	
	wp_SetTrayIcon(true)	
  return

WindowPadX_Init(IniPath="")
{
    global WINDOWPADX_INI_PATH
    ;
    ; Init icons and tray menu.
    ;
    if A_IsCompiled  ; Load icons from my custom WindowPadX.exe.
    {
        ; Default icon is 32x32, so doesn't look good in the tray.
        Menu, Tray, Icon, %A_ScriptFullPath%, 2
    }
    else if (A_LineFile = A_ScriptFullPath)
    {   ; Set the tray icon, but only if not included in some other script.
        wp_SetTrayIcon(true)
        ; Use OnMessage to catch "Suspend Hotkeys" or "Pause Script"
        ; so the "disabled" icon can be used.
        OnMessage(0x111, "WM_COMMAND")
    }
        
    Menu, Tray, NoStandard
    Menu, Tray, MainWindow
    Menu, Tray, Add, &Debug, TrayDebug
    ifExist, %A_ScriptDir%\WindowPadX.html
    {
        Menu, Tray, Add, &Help, TrayHelp
        Menu, Tray, Add
    }
    Menu, Tray, Add, &Reload, TrayReload
    if !A_IsCompiled
    {
        Menu, Tray, Add, &Edit Source, TrayEdit
    }
    Menu, Tray, Add, Edit &Configuration, TrayEditConfig
    Menu, Tray, Add
    Menu, Tray, Add, &Suspend, TraySuspend
    Menu, Tray, Add, E&xit, TrayExit
    Menu, Tray, Default, &Debug    
    ;
    ; Load settings.
    ;
    if IniPath =
        Loop, %A_LineFile%\..\WindowPadX.ini
            IniPath := A_LoopFileFullPath
    ifNotExist %IniPath%
    {
        IniPath = %A_ScriptDir%\WindowPadX.ini
        FileInstall, WindowPadX.Default.ini, %IniPath%
    }
    WINDOWPADX_INI_PATH := IniPath
    WindowPadX_LoadSettings(IniPath)
	
}

WindowPadX_LoadSettings(ininame)
{
    local v
    
    ; Misc Options
    IniRead, v, %ininame%, Options, TitleMatchMode, %A_TitleMatchMode%
    SetTitleMatchMode, %v%
    
    ; Hotkeys: Exclude Windows
    v := wp_INI_GetList(ininame, "Exclude Windows", "Window")
    Loop, Parse, v, `n
        GroupAdd, HotkeyExclude, %A_LoopField%
    
    ; Read the Hotkeys section in.
    v := wp_INI_ReadSection(ininame, "Hotkeys")
    ; Replace the first = with ::.
    ; ('=' is required for WritePrivateProfileSection to work properly.)
    v := RegExReplace(v, "m`a)^(.*?)=", "$1::")
    Hotkey, IfWinNotActive, ahk_group HotkeyExclude
    Hotkey_Params(v)
    
    ; Gather: Exclude Windows
    v := wp_INI_GetList(ininame, "Gather: Exclude Windows", "Window")
    Loop, Parse, v, `n
        GroupAdd, GatherExclude, %A_LoopField%

    ; Gather: Exclude Processes
    ProcessGatherExcludeList := wp_INI_GetList(ininame
        , "Gather: Exclude Processes", "Process", ",")
}

wp_INI_GetList(ininame, Section, Key, Delim="`n")
{
    v := wp_INI_ReadSection(ininame, Section)
    Loop, Parse, v, `n
    {
        pos := InStr(A_LoopField, "=")
        if (pos && SubStr(A_LoopField,1,pos-1) = Key)
            list .= (list ? Delim : "") . SubStr(A_LoopField, pos+1)
    }
    return list
}

wp_INI_ReadSection(Filename, Section)
{
    char_type := A_IsUnicode ? "UShort" : "UChar"
    char_size := A_IsUnicode ? 2 : 1
    
    ; Expand relative paths, since GetPrivateProfileSection only searches %A_WinDir%.
    Loop, %Filename%, 0
        Filename := A_LoopFileLongPath
    
    VarSetCapacity(buf, 0x7FFF*char_size, 0)

    len := DllCall("GetPrivateProfileSection"
        , "uint", &Section, "uint", &buf, "uint", 0x7FFF, "uint", &Filename)
    
    VarSetCapacity(text, len*char_size), p := &buf
    ; For each null-terminated substring,
    while (s := DllCall("MulDiv", "int", p, "int", 1, "int", 1, "str"))
        ; append it to the output and advance to the next substring.
        text .= s "`n",  p += (StrLen(s)+1)*char_size
    
    ; Strip the trailing newline
    text := SubStr(text, 1, -1)
    
    ; Windows Me/98/95:
    ;   The returned string includes comments.
    ;
    ; This removes comments. Also, I'm not sure if leading/trailing space is
    ; automatically removed on Win9x, so the regex removes that too.
    if A_OSVersion in WIN_ME,WIN_98,WIN_95
        text := RegExReplace(text, "m`n)^[ `t]*(?:;.*`n?|`n)|^[ `t]+|[ `t]+$")
    
    return text
}


; Tray menu subroutines. May also be assigned to hotkeys in WindowPadX.ini.
TrayDebug:
    ListHotkeys
  return
TrayHelp:
    Run, %A_ScriptDir%\WindowPadX.html
  return
TrayReload:
    Reload
  return
TrayEdit:
    Edit
  return
TrayEditConfig:
    RegRead, Editor, HKCR, AutoHotkeyScript\Shell\Edit\Command
    StringReplace, Editor, Editor, "`%1",
    Trim(Editor)
    if (Editor = )
        Editor = notepad
    Run, %Editor% "%A_ScriptDir%\WindowPadX.ini"
    return
TraySuspend:
    WM_COMMAND(65305,0)
    Suspend
    Menu, Tray, % A_IsSuspended ? "Check" : "Uncheck", &Suspend
  return
TrayExit:


    wp_Restore()
    ExitApp


;
; Hotkeys: Activate hotkeys defined in INI section [Hotkeys: %section%].
;
Hotkeys(section, options)
{
    local this_hotkey, section_var, hotkeys, wait_for_keyup, m, m1, pos, k
    static key_regex = "^(?:.* & )?[#!^+&<>*~$]*(.+)"
    
    this_hotkey := A_ThisHotkey
    
    if !section
        goto HC_SendThisHotkeyAndReturn
    
    pos := RegExMatch(options, "i)(?<=\bD)\d*\.?\d*", m)
    if pos
    {
        options := SubStr(options, 1, pos-2) . SubStr(options, pos+StrLen(m))
        if (m+0 = "")
            m := 0.1
        Input, k, L1 T%m%
        if ErrorLevel != Timeout
        {
            gosub HC_SendThisHotkey
            Send %k%
            return
        }
    }
    
    section_var := RegExReplace(section, "[^\w#@$]", "_")
    hotkeys := Hotkeys_%section_var%
    
    if hotkeys =
    {
        ; Load each hotkeys section on first use. Since the ini file may be
        ; edited between enabling and disabling the hotkeys, loading them
        ; each and every time would be hazardous.
        hotkeys := wp_INI_ReadSection(WINDOWPADX_INI_PATH, "Hotkeys: " section)
        if hotkeys =
            goto HC_SendThisHotkeyAndReturn
        
        ; key=command  ->  key::command
        hotkeys := RegExReplace(hotkeys, "m`a)^(.*?)=", "$1::")
        
        Hotkeys_%section_var% := hotkeys
    }
        
    ; If Options were omitted and this is a key-down hotkey,
    ; automatically disable the hotkeys when the key is released.
    if (wait_for_keyup := (options="" && SubStr(this_hotkey,-2) != " up"))
        options = On ; Explicit "on" in case hotkey exists but is disabled.
    
    Hotkey, IfWinNotActive, ahk_group HotkeyExclude
    Hotkey_Params(hotkeys, options)
    
    if (wait_for_keyup)
    {
        if (!RegExMatch(this_hotkey, key_regex, m) || GetKeyState(m1)="") {
            MsgBox, % "Error retrieving primary key of hotkey in Hotkeys().`n"
                    . "`tHotkey: " this_hotkey "`n"
                    . "`tResult: " m1
                    . "`nPlease inform Lexikos. Tip: Press Ctrl+C to copy this message."
            return
        }
        
        KeyWait, %m1%
        
        Hotkey_Params(hotkeys, "Off")

        ; A_ThisHotkey: "The key name of the *most recently executed* hotkey"
        ;if(some other hotkey was executed during KeyWait)
        if (this_hotkey = A_ThisHotkey)
            goto HC_SendThisHotkeyAndReturn
    }
    return

HC_SendThisHotkey:
HC_SendThisHotkeyAndReturn:
    if ! InStr(this_hotkey, "~")
        if (RegExMatch(this_hotkey, key_regex, m) && GetKeyState(m1)!="") {
            Hotkey, %this_hotkey%, Off
            Send {Blind}{%m1%}
            Hotkey, %this_hotkey%, On
        }
    return
}


;
; Commands implemented as labels for simplicity.
;
Send:
    Send, %Params%
    return
Minimize:
    if wp_WinExist(Params)
        PostMessage, 0x112, 0xF020  ; WM_SYSCOMMAND, SC_MINIMIZE
    return
Unminimize:
    if wp_WinLastMinimized()
        WinRestore
    return
Restore:
    if wp_WinExist(Params)
        WinRestore
    return
    
;
; Internal Functions
;
; Execute a pseudo-command with params.
wp_ExecLine(cmdline)
{
    if RegExMatch(cmdline, "^\s*(?<Name>\S+?)(?:[, `t]\s*(?<Params>.*?))?\s*$", a)
    {
        global Params := aParams  ; Global for use by label-based pseudo-commands.
        if (n := IsFunc(aName)) && n <= 6
        {   ; %aName% is a function with up to 5 params.
            if n > 2
            {   ; Two or more required parameters.
                if RegExMatch(aParams, "^(?:[^,]*,){1," n-2 "}\s*", aParams)
                    StringSplit, p, aParams, `,, %A_Space%%A_Tab%
                else ; no comma
                    p0 := 1
                p%p0% := SubStr(Params, StrLen(aParams) + 1)
            }
            else
                ; At most one required parameter.
                p1 := Params
            ; Call function with values supplied for only the required params.
            ; This allows commas to be treated literally in the last param.
            return %aName%(p1, p2, p3, p4, p5)
        }
        else
            ; aName is a label or invalid.
            gosub %aName%
    }
}

; Hotkey_Params( line [, Options ] )
;   Associates a hotkey with a parameter string.
;
; Expects a newline(`n)-delimited list of hotkeys in the form:
;   Hotkey:: LabelName, Params
;
; Note:
;   - Spaces are optional.
;   - As with hotkey labels, there should be no space between 'Hotkey' and '::'.
;   - Unlike the Hotkey command, LabelName MUST NOT be omitted.
;   - Params MUST NOT contain a newline character (`n).
;   - Params may contain zero or more commas.
;   - , (comma) is supported as a hotkey.
;   - Unlike the Hotkey command, 'Toggle' should be specified in the Options, not as a label.
;
; Returns the number of hotkeys successfully enabled/disabled.
;
Hotkey_Params(line, Options="")
{
    static List ; List of hotkeys and associated labels + parameters.
        , sCmdLine ; temp var used by hotkey subroutine.
    
    count = 0
    
    ; Note: The parsing loop operates on a temporary copy of 'line',
    ;       so 'line' can be (and is) reused within the loop.
    
    Loop, Parse, line, `n, %A_Space%%A_Tab%
    {
        ; Clear ErrorLevel in case UseErrorLevel is (not) specified.
        ErrorLevel =

        if ! RegExMatch(A_LoopField, "^\s*(?<Hotkey>.+?)::\s*(?<Name>.+?)(?:[, `t]\s*(?<Params>.*?))?\s*$", line)
            continue
        
        if !(IsLabel(lineName) || IsFunc(lineName))
            continue
        
        if Options = Toggle ; Not supported as an option (must be Label.)
        {
            ; Toggle hotkey.  If it doesn't exist, the next line will enable it.
            Hotkey, %lineHotkey%, Toggle, UseErrorLevel
            ; Ensure the hotkey will execute the correct label.
            Hotkey, %lineHotkey%, hp_ExecuteHotkeyWithParams, UseErrorLevel
        } else
            Hotkey, %lineHotkey%, hp_ExecuteHotkeyWithParams, %Options%
        
        ; Check ErrorLevel in case UseErrorLevel was specified.
        if ErrorLevel
            continue
        
        ; Rebuild line to remove whitespace.
        line := lineHotkey "::" lineName "," lineParams
        
        ; Update an existing hotkey's label + params,
        temp := RegExReplace(List, "m`n)^\Q" lineHotkey "\E::.*$", line, repl, 1)
        if (repl > 0)
            List := temp
        else    ; or add a new hotkey to the list.
            List .= (List ? "`n" : "") . line

        count += 1
    }
    return count

hp_ExecuteHotkeyWithParams:
    if RegExMatch(List, "m`n)^\Q" A_ThisHotkey "\E::\K.*", sCmdLine)
        wp_ExecLine(sCmdLine)
return
}


;
; Tray Icon Override:
;   Provides a way to customize the icons without compiling the script.
;

WM_COMMAND(wParam, lParam)
{
    static IsPaused, IsSuspended
    Critical
    id := wParam & 0xFFFF
    if id in 65305,65404,65306,65403
    {  ; "Suspend Hotkeys" or "Pause Script"
        if id in 65306,65403  ; pause
            IsPaused := ! IsPaused
        else  ; at this point, A_IsSuspended has not yet been toggled.
            IsSuspended := ! A_IsSuspended
        wp_SetTrayIcon(!(IsPaused or IsSuspended))
    }
}

wp_SetTrayIcon(is_enabled)
{
    icon := is_enabled ? "tray.ico" : "disabled.ico"
    icon = %A_ScriptDir%\icons\%icon%

    ; avoid an error message if the icon doesn't exist
    IfExist, %icon%
        Menu, TRAY, Icon, %icon%,, 1
}

/* my Block Autohotkey sctipts
==============================
*/

Alt & RButton::
CoordMode, Mouse  ; Switch to screen/absolute coordinates.
MouseGetPos, EWD_MouseStartX, EWD_MouseStartY, EWD_MouseWin
WinGetPos, EWD_OriginalPosX, EWD_OriginalPosY,,, ahk_id %EWD_MouseWin%
WinGet, EWD_WinState, MinMax, ahk_id %EWD_MouseWin% 
if EWD_WinState = 0  ; Only if the window isn't maximized 
    SetTimer, EWD_WatchMouse, 10 ; Track the mouse as the user drags it.
return

EWD_WatchMouse:
GetKeyState, EWD_LButtonState, RButton, P
if EWD_LButtonState = U  ; Button has been released, so drag is complete.
{
    SetTimer, EWD_WatchMouse, off
    return
}
GetKeyState, EWD_EscapeState, Escape, P
if EWD_EscapeState = D  ; Escape has been pressed, so drag is cancelled.
{
    SetTimer, EWD_WatchMouse, off
    WinMove, ahk_id %EWD_MouseWin%,, %EWD_OriginalPosX%, %EWD_OriginalPosY%
    return
}
; Otherwise, reposition the window to match the change in mouse coordinates
; caused by the user having dragged the mouse:
CoordMode, Mouse
MouseGetPos, EWD_MouseX, EWD_MouseY
WinGetPos, EWD_WinX, EWD_WinY,,, ahk_id %EWD_MouseWin%
SetWinDelay, -1   ; Makes the below move faster/smoother.
WinMove, ahk_id %EWD_MouseWin%,, EWD_WinX + EWD_MouseX - EWD_MouseStartX, EWD_WinY + EWD_MouseY - EWD_MouseStartY
EWD_MouseStartX := EWD_MouseX  ; Update for the next timer-call to this subroutine.
EWD_MouseStartY := EWD_MouseY
return


;___________________________________________
Transparency_Up:
WinGet, Transparency, Transparent, A
if varExist(Transparency) = 0 
	Transparency = 255
Transparency += Transparency_Step
IfGreater Transparency, 255, SetEnv Transparency,255
WinSet, Transparent, %Transparency%, A
Gosub, Transparency_ShowBars
return

Transparency_Down:
WinGet, Transparency, Transparent, A
vbTransparency := varExist(Transparency)
ifEqual,vbTransparency,0
{
Transparency := 255
; msgbox vbTransparency:%vbTransparency%|Transparency:%Transparency%
}
ifEqual,vbTransparency,2
{
Transparency := 255
; msgbox vbTransparency:%vbTransparency%|Transparency:%Transparency%
}
; msgbox TELO %Transparency% and %vbTransparency%
Transparency -= Transparency_Step
IfLess Transparency, 1, SetEnv Transparency,1
WinSet, Transparent, %Transparency%, A
Gosub, Transparency_ShowBars
return

Transparency_ShowBars:
; To prevent the "flashing" effect, only create the bar window if it
; doesn't already exist:

IfWinNotExist, Transparency
Progress, %Transparency_BarOptions%, , , Transparency
IfWinNotExist, Transparency
{
	; Calculate position here in case screen resolution changes while
	; the script is running:

	if Transparency_PosY < 0
	{
		; Create the bar :
		WinGetPos, , Transparency_Posy, , , Transparency
		Transparency_Posy -= %Transparency_Thick%
		Progress, %Transparency_BarOptions% Y%Transparency_Posy%, , ,
	}
	else
		Progress, %Transparency_BarOptionsTransparency%, , ,
}

Progress, % Transparency/2.55
SetTimer, Transparency_BarOff, %Transparency_DisplayTime%
return

Transparency_BarOff:
SetTimer, Transparency_BarOff, off
Progress, Off
return

varExist(ByRef v) { ; Requires 1.0.46+
   return &v = &n ? 0 : v = "" ? 2 : 1 
}
/* END my Block Autohotkey sctipts
==============================
*/

; http://habrahabr.ru/post/24652/
LWin & RButton::
CoordMode, Mouse ; Switch to screen/absolute coordinates.
MouseGetPos, SWM_MouseStartX, SWM_MouseStartY, SWM_MouseWin
WinGetPos, SWM_WinX, SWM_WinY, SWM_WinW, SWM_WinH, ahk_id %SWM_MouseWin%
WinGetClass, SWM_Win_Class, ahk_id %SWM_MouseWin%
If SWM_Win_Class = ProgMan
Return
WinGet, State, MinMax, ahk_id %SWM_MouseWin%
If State = 1
{
SplashImage,, W160 H26 B1 FM8 WM400 CT000080,,  ,, Calibri
SetTimer, Remove_Splash, 600
Return
}
GetKeyState, SMW_LCtrlState, LCtrl
if SMW_LCtrlState=D
{
WinClose, ahk_id %SWM_MouseWin%
return
}
SWM_ResizeTypeX=0
SWM_ResizeTypeY=0
if (SWM_MouseStartX < SWM_WinX+SWM_WinW/2)
SWM_ResizeTypeX=1
if (SWM_MouseStartY < SWM_WinY+SWM_WinH/2)
SWM_ResizeTypeY=1
SetTimer, SWM_WatchMouse_Resize, 10
return

SWM_WatchMouse_Move:
GetKeyState, SMW_LButtonState, LButton, P
if SMW_LButtonState = U
{
SetTimer, SWM_WatchMouse_Move, off
return
}
Gosub SWM_GetMouseAndWindowPos
SWM_WinX += %SWM_DeltaX%
SWM_WinY += %SWM_DeltaY%
SetWinDelay, -1
WinMove, ahk_id %SWM_MouseWin%,, %SWM_WinX%, %SWM_WinY%
return

SWM_WatchMouse_Resize:
GetKeyState, SMW_RButtonState, RButton, P
if SMW_RButtonState = U
{
SetTimer, SWM_WatchMouse_Resize, off
return
}
Gosub SWM_GetMouseAndWindowPos
if SWM_ResizeTypeX
{
SWM_WinX += %SWM_DeltaX%
SWM_WinW -= %SWM_DeltaX%
}
else
SWM_WinW += %SWM_DeltaX%
if SWM_ResizeTypeY
{
SWM_WinY += %SWM_DeltaY%
SWM_WinH -= %SWM_DeltaY%
}
else
SWM_WinH += %SWM_DeltaY%
SetWinDelay, -1
WinMove, ahk_id %SWM_MouseWin%,, %SWM_WinX%, %SWM_WinY%, %SWM_WinW%, %SWM_WinH%
return

SWM_GetMouseAndWindowPos:
CoordMode, Mouse
MouseGetPos, SWM_MouseX, SWM_MouseY
SWM_DeltaX = %SWM_MouseX%
SWM_DeltaX -= %SWM_MouseStartX%
SWM_DeltaY = %SWM_MouseY%
SWM_DeltaY -= %SWM_MouseStartY%
SWM_MouseStartX = %SWM_MouseX%
SWM_MouseStartY = %SWM_MouseY%
WinGetPos, SWM_WinX, SWM_WinY, SWM_WinW, SWM_WinH, ahk_id %SWM_MouseWin%
return

#LButton::
CoordMode, Mouse
MouseGetPos, EWD_MouseStartX, EWD_MouseStartY, EWD_MouseWin
WinGetClass, EWD_Win_Class, ahk_id %EWD_MouseWin%
If EWD_Win_Class = ProgMan
Return
WinGet, State, MinMax, ahk_id %EWD_MouseWin%
If State = 1
{
SplashImage,, W160 H27 B1 FM8 WM400 CT000080,,  ,, Calibri
SetTimer, Remove_Splash, 600
Return

Remove_Splash:
SetTimer, Remove_Splash, Off
SplashImage, Off
Return
}
WinGetPos, EWD_OriginalPosX, EWD_OriginalPosY,,, ahk_id %EWD_MouseWin%
SetTimer, EWD_WatchMouse_two, 10
Return

EWD_WatchMouse_two:
EWD_Work = 1
GetKeyState, EWD_LButtonState, LButton, P
If EWD_LButtonState = U
{
SetTimer, EWD_WatchMouse_two, off
EWD_Work =
Return
}
GetKeyState, EWD_EscapeState, Escape, P
If EWD_EscapeState = D
{
SetTimer, EWD_WatchMouse_two, off
EWD_Work =
WinMove, ahk_id %EWD_MouseWin%,, %EWD_OriginalPosX%, %EWD_OriginalPosY%
Return
}
CoordMode, Mouse
MouseGetPos, EWD_MouseX, EWD_MouseY
WinGetPos, EWD_WinX, EWD_WinY,,, ahk_id %EWD_MouseWin%
SetWinDelay, -1
WinMove, ahk_id %EWD_MouseWin%,, EWD_WinX + EWD_MouseX - EWD_MouseStartX, EWD_WinY + EWD_MouseY - EWD_MouseStartY
EWD_MouseStartX := EWD_MouseX
EWD_MouseStartY := EWD_MouseY
Return


